@@ -109,12 +109,9 @@ Handle<String> ReadLineEditor::Prompt(const char* prompt) {
     Unlocker unlock(Isolate::GetCurrent());
     result = readline(prompt);
   }
-  if (result != NULL) {
-    AddHistory(result);
-  } else {
-    return Handle<String>();
-  }
-  return String::New(result);
+  if (result == NULL) return Handle<String>();
+  AddHistory(result);
+  return String::NewFromUtf8(isolate_, result);
 }
 
 
@@ -150,24 +147,29 @@ char* ReadLineEditor::CompletionGenerator(const char* text, int state) {
   static Persistent<Array> current_completions;
   Isolate* isolate = read_line_editor.isolate_;
   Locker lock(isolate);
+  HandleScope scope(isolate);
+  Handle<Array> completions;
   if (state == 0) {
-    HandleScope scope;
-    Local<String> full_text = String::New(rl_line_buffer, rl_point);
-    Handle<Array> completions =
-        Shell::GetCompletions(isolate, String::New(text), full_text);
-    current_completions = Persistent<Array>::New(isolate, completions);
+    Local<String> full_text = String::NewFromUtf8(isolate,
+                                                  rl_line_buffer,
+                                                  String::kNormalString,
+                                                  rl_point);
+    completions = Shell::GetCompletions(isolate,
+                                        String::NewFromUtf8(isolate, text),
+                                        full_text);
+    current_completions.Reset(isolate, completions);
     current_index = 0;
+  } else {
+    completions = Local<Array>::New(isolate, current_completions);
   }
-  if (current_index < current_completions->Length()) {
-    HandleScope scope;
-    Handle<Integer> index = Integer::New(current_index);
-    Handle<Value> str_obj = current_completions->Get(index);
+  if (current_index < completions->Length()) {
+    Handle<Integer> index = Integer::New(isolate, current_index);
+    Handle<Value> str_obj = completions->Get(index);
     current_index++;
     String::Utf8Value str(str_obj);
     return strdup(*str);
   } else {
-    current_completions.Dispose(isolate);
-    current_completions.Clear();
+    current_completions.Reset();
     return NULL;
   }
 }
